<?php

namespace Drupal\entity_extender\Plugin\EntityTypeExtender;

/**
 * The base entity handler definition.
 */
abstract class EntityHandlerDefinition {

  /**
   * The handler class the entity type has by default.
   *
   * @var string
   */
  protected string $defaultClass;

  /**
   * The handler class to set to the entity type.
   *
   * NOTE: this class must extend the default one.
   *
   * @var string
   */
  protected string $replacingClass;

  /**
   * {@inheritdoc}
   */
  public function __construct(string $default_class, string $replacing_class) {
    $this->defaultClass = $default_class;
    $this->replacingClass = $replacing_class;
  }

  /**
   * Returns the type ID of the entity handler.
   *
   * @return string
   *   The entity handler ID.
   */
  abstract public function getType(): string;

  /**
   * Returns the trait the handler class must user.
   *
   * @return string
   *   The trait the handler class must user.
   *
   * @see \Drupal\entity_extender\Plugin\EntityTypeExtender\EntityHandlerTrait
   */
  abstract public function getRequiredTrait(): string;

  /**
   * Returns the interfaces the extender could implement.
   *
   * @return string[]
   *   The interfaces to implement by the entity extender plugin
   *   to opt-in to a specific entity handling.
   *
   * @see \Drupal\entity_extender\Plugin\EntityExtender\EntityExtenderHandlerInterface
   */
  abstract public function getEntityExtenderHandlersInterfaces(): array;

  /**
   * {@inheritdoc}
   */
  public function getDefaultClass(): string {
    return $this->defaultClass;
  }

  /**
   * {@inheritdoc}
   */
  public function getReplacingClass(): string {
    return $this->replacingClass;
  }

}
