<?php

namespace Drupal\entity_extender\Plugin\EntityTypeExtender;

use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\entity_extender\Access\AccessResultAllowedWithoutCoreAccessChecks;
use Drupal\entity_extender\Plugin\EntityExtender\EntityAccessInterface;

/**
 * The extended access handler for entities.
 */
trait EntityHandlerTraitAccess {

  use EntityHandlerTrait;

  /**
   * {@inheritdoc}
   */
  public function access(EntityInterface $entity, $operation, AccountInterface $account = NULL, $return_as_object = FALSE) {
    if ($extender = $this->entityExtenderManager->getExtenderIfMemberOf(EntityAccessInterface::class, $entity)) {
      \assert($extender instanceof EntityAccessInterface);
      $account = $this->prepareUser($account);
      $access = static::useMemo(static fn (): AccessResultInterface => $extender->access($operation, $account), $operation, $account);
      \assert($access instanceof AccessResultInterface);

      // The "allowed" and "neutral" states let us proceed to core's access
      // checks. Only forbiddance terminates access checks immediately.
      if ($access->isForbidden()) {
        return $return_as_object ? $access : FALSE;
      }

      if ($access instanceof AccessResultAllowedWithoutCoreAccessChecks) {
        return $return_as_object ? $access : TRUE;
      }
    }

    return parent::access($entity, $operation, $account, $return_as_object);
  }

}
